#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Bootstrap module
"""

__all__ = ['require', 'loader']

import os
import sys

try:
    version_info = sys.version_info
except AttributeError:
    version_info = (1, 5)  # 1.5 or older

KNOWN_PYTHONS = [
    ((2, 1), 'python2.1'),
    ((2, 1), 'pythonw2.1'),
    ((2, 2), 'python2.2'),
    ((2, 2), 'pythonw2.2'),
    ((2, 3), 'python2.3'),
    ((2, 3), 'pythonw2.3'),
    ((2, 4), 'python2.4'),
    ((2, 4), 'pythonw2.4'),
    ((2, 5), 'python2.5'),
    ((2, 5), 'pythonw2.5'),
    ((2, 6), 'python2.6'),
    ((2, 6), 'pythonw2.6'),
    ((2, 7), 'python2.7'),
    ((2, 7), 'pythonw2.7'),
    ((3, 0), 'python3.0'),
    ((3, 0), 'pythonw3.0'),
    ((3, 1), 'python3.1'),
    ((3, 1), 'pythonw3.1'),
    ]


def loader(version=(2, 5), script='__auto__'):
    """Load module with proper python interpreter"""

    if script == '__auto__':
        for ext in ['py', 'pyw', 'pyc', 'pyo']:
            script = os.path.splitext(sys.argv[0])[0] + os.extsep + ext
            if os.path.isfile(script):
                break

        if script == '__auto__' or not os.path.isfile(script):
            raise RuntimeError("Can't guess target executable script")

    def _runner(python, script):
        try:
            if sys.platform == 'win32':
                argv = ['"%s"' % arg for arg in sys.argv[1:]]
            else:
                argv = sys.argv[1:]
            if script:
                os.execvp(python, [python, script] + argv)
            else:
                os.execvp(python, [python] + argv)
        except OSError:
            pass

    if version_info >= version:
        _runner(sys.executable, script)
    else:
        for (ver, python) in KNOWN_PYTHONS:
            if ver >= version:
                _runner(python, script)

    raise RuntimeError('Python %d.%d or greater required\n' % version)


def require(version=(2, 5)):
    """Reinvoke facility"""

    # Algorithm borrowed from bzr vcs

    REINVOKE = '__BOOTSTRAP_REINVOKE'

    if version_info < version:
        if REINVOKE not in os.environ:
            os.putenv(REINVOKE, '1')
            for (ver, python) in KNOWN_PYTHONS:
                if ver >= version:
                    try:
                        if sys.platform == 'win32':
                            argv = ['"%s"' % arg for arg in sys.argv]
                        else:
                            argv = sys.argv[:]
                        os.execvp(python, [python] + argv)
                    except OSError:
                        pass
        sys.stderr.write('Sorry, Python %d.%d or greater required\n'
                          % version)
        sys.exit(1)

    if REINVOKE in os.environ:
        del os.environ[REINVOKE]

    if hasattr(os, 'unsetenv'):
        os.unsetenv(REINVOKE)


# vim: set fileencoding=utf-8
# vim: et ts=4 sts=4 sw=4
